/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifndef __TAPIOCAUI_COMMUNICATION_H__
#define __TAPIOCAUI_COMMUNICATION_H__

#include <gtk/gtkwidget.h>
#include <gtk/gtkdialog.h>
#include "tapiocaui_db.h"

G_BEGIN_DECLS

#define TAPIOCAUI_TYPE_COMMUNICATION          (tapiocaui_communication_get_type ())
#define TAPIOCAUI_COMMUNICATION(o)            (G_TYPE_CHECK_INSTANCE_CAST ((o), TAPIOCAUI_TYPE_COMMUNICATION, TapiocaUICommunication))
#define TAPIOCAUI_COMMUNICATION_CLASS(k)      (G_TYPE_CHECK_CLASS_CAST((k), TAPIOCAUI_TYPE_COMMUNICATION, TapiocaUICommunicationClass))
#define TAPIOCAUI_IS_COMMUNICATION(o)         (G_TYPE_CHECK_INSTANCE_TYPE ((o), TAPIOCAUI_TYPE_COMMUNICATION))
#define TAPIOCAUI_IS_COMMUNICATION_CLASS(k)   (G_TYPE_CHECK_CLASS_TYPE ((k), TAPIOCAUI_TYPE_COMMUNICATION))
#define TAPIOCAUI_COMMUNICATION_GET_CLASS(o)  (G_TYPE_INSTANCE_GET_CLASS ((o), TAPIOCAUI_TYPE_COMMUNICATION, TapiocaUICommunicationClass))

typedef enum 
{
    TAPIOCAUI_CALL_IDLE,
    TAPIOCAUI_CALL_HOLD,
    TAPIOCAUI_CALL_INCOMING,
    TAPIOCAUI_CALL_RINGING,
    TAPIOCAUI_CALL_ANSWERED
}TapiocaUICommunicationStatus;

typedef struct _TapiocaUICommunication        TapiocaUICommunication;
typedef struct _TapiocaUICommunicationClass   TapiocaUICommunicationClass;
typedef struct _TapiocaUICommunicationPrivate TapiocaUICommunicationPrivate;

struct _TapiocaUICommunicationClass
{
    GtkWindowClass parent_class;
  
    void (* volume_change)      (guint volume);
    void (* input_sound_mute)   (void);
    void (* input_sound_unmute) (void);
    void (* call_make)          (const gchar *user_id);
    void (* call_answer)        (const gchar *user_id);
    void (* call_reject)        (const gchar *user_id);
    void (* call_disconect)     (const gchar *user_id);
    void (* call_hold)          (const gchar *user_id);
    void (* call_unhold)        (const gchar *user_id);
    void (* send_message)       (const gchar *message);
    void (* quit)               (void);
};

struct _TapiocaUICommunication
{
    GtkWindow parent;

    TapiocaUICommunicationPrivate *priv;
};

GType                     tapiocaui_communication_get_type      (void);
GtkWidget                *tapiocaui_communication_new           (const gchar                 *user,
                                                                 TapiocaUIBuddyData          *buddy);
const TapiocaUIBuddyData *tapiocaui_communication_get_buddy     (TapiocaUICommunication      *self);
void                      tapiocaui_communication_set_status    (TapiocaUICommunication      *self,
                                                                 TapiocaUICommunicationStatus status);
void                      tapiocaui_communication_set_volume    (TapiocaUICommunication      *self,
                                                                 guint                        volume);
void                      tapiocaui_communication_append_message (TapiocaUICommunication     *self,
                                                                  guint                       timestamp,
                                                                  const gchar                *user,
                                                                  const gchar                *msg);
guint                     tapiocaui_communication_get_volume    (TapiocaUICommunication      *self);
void                      tapiocaui_communication_closed_win_message (TapiocaUICommunication *self,
                                                                      const gchar            *user);

G_END_DECLS

#endif
