/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifndef __TAPIOCAUI_DB_H__
#define __TAPIOCAUI_DB_H__

#include <glib.h>
#include <glib/gtypes.h>
#include <glib/glist.h>
#include <glib-object.h>
#include <tapioca/core/tapioca_enums.h>

G_BEGIN_DECLS

#define TAPIOCAUI_TYPE_DB         (tapiocaui_db_get_type ())
#define TAPIOCAUI_DB(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), TAPIOCAUI_TYPE_DB, TapiocaUIDB))
#define TAPIOCAUI_DB_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), TAPIOCAUI_TYPE_DB, TapiocaUIDBClass))
#define TAPIOCAUI_IS_DB(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), TAPIOCAUI_TYPE_DB))
#define TAPIOCAUI_IS_DB_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), TAPIOCAUI_TYPE_DB))
#define TAPIOCAUI_DB_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TAPIOCAUI_TYPE_DB, TapiocaUIDBClass))

typedef enum 
{
    TAPIOCAUI_DB_FIELD_ID = 0,
    TAPIOCAUI_DB_FIELD_NAME,
    TAPIOCAUI_DB_FIELD_GROUP,
    TAPIOCAUI_DB_FIELD_PROXY_PASSWD,
    TAPIOCAUI_DB_FIELD_PHOTO,
    TAPIOCAUI_DB_FIELD_ACCOUNT_TYPE
} TapiocaUIDBField;

typedef enum
{
    TAPIOCAUI_DB_ENTRY_BUDDY = 0,
    TAPIOCAUI_DB_ENTRY_ACCOUNT,
    TAPIOCAUI_DB_ENTRY_LAST
} TapiocaUIDBEntryType;

typedef struct _TapiocaUIDB             TapiocaUIDB;
typedef struct _TapiocaUIDBClass        TapiocaUIDBClass;
typedef struct _TapiocaUIDBPrivate      TapiocaUIDBPrivate;
typedef struct _TapiocaUIDBEntryPrivate TapiocaUIDBEntryPrivate;
typedef struct _TapiocaUIBuddyData      TapiocaUIBuddyData;
typedef struct _TapiocaUIBuddyPresence  TapiocaUIBuddyPresence;

struct _TapiocaUIBuddyData 
{
    gchar *id;
    gchar *name;
    gchar *photo;    
    gchar *account_type;
    gchar *group;

    GHashTable *jids;

    TapiocaContactListSubscriptionState  subscription_state;
    
    TapiocaUIDBEntryPrivate *priv;
};

struct _TapiocaUIBuddyPresence
{
    guint priority;
    guint status;
    gchar *status_desc;
    gboolean voice_cap;
};

struct _TapiocaUIDBClass 
{   
    GObjectClass parent_class;
    
    void (* db_changed)       (void);
    void (* db_buddy_added)   (gpointer id);
    void (* db_buddy_removed) (gpointer id);
};

struct _TapiocaUIDB 
{
    GObject parent;
    
    TapiocaUIDBPrivate *priv;
};

GType               tapiocaui_db_get_type              (void);
TapiocaUIDB        *tapiocaui_db_new                   (const gchar         *file_name);
gboolean            tapiocaui_db_load                  (TapiocaUIDB         *self);
void                tapiocaui_db_commit                (TapiocaUIDB         *self);
void                tapiocaui_db_notify_change         (TapiocaUIDB         *self);
TapiocaUIBuddyData *tapiocaui_db_entry_new             (TapiocaUIDBEntryType type);
void                tapiocaui_db_entry_free            (TapiocaUIBuddyData  *entry);
TapiocaUIBuddyData *tapiocaui_db_find_buddy_by_id      (TapiocaUIDB         *self,
                                                        TapiocaUIDBEntryType type,
                                                        const gchar         *id);
void                tapiocaui_db_insert_buddy          (TapiocaUIDB         *self, 
                                                        TapiocaUIBuddyData  *buddy);
void                tapiocaui_db_remove_buddy          (TapiocaUIDB         *self,
                                                        TapiocaUIBuddyData  *buddy);
void                tapiocaui_db_sync_buddy            (TapiocaUIDB         *self,
                                                        TapiocaUIBuddyData  *buddy);
GList              *tapiocaui_db_get_buddy_list        (TapiocaUIDB         *self,
                                                        TapiocaUIDBEntryType type);
void                tapiocaui_db_free                  (TapiocaUIDB         *self); 
GList              *tapiocaui_db_filter                (TapiocaUIDB         *self,
                                                        TapiocaUIDBEntryType type,
                                                        TapiocaUIDBField     propid,
                                                        const gchar         *value);
gboolean            tapiocaui_db_voice_cap_hash_filter (gpointer             key,
                                                        gpointer             value,
                                                        gpointer             user_data);

G_END_DECLS

#endif
