/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtktreeview.h>
#include <gtk/gtkliststore.h>
#include <gtk/gtkcellrenderer.h>
#include <gtk/gtkcellrenderertext.h>
#include <gtk/gtkcellrendererpixbuf.h>
#include <gtk/gtktreeselection.h>
#include <tapioca/core/tapioca_enums.h>

#include "tapiocaui_buddy_list.h"
#include "tapiocaui_cell_renderer_pixbuf.h"
#include "tapiocaui_db.h"
#include "tapiocaui_marshal.h"
#include "tapiocaui_utils.h"

#include <string.h>

enum
{
    PROP_0,
    PROP_DB
};

enum
{
    BUDDY_ACTIVATED,
    BUDDY_SELECTION_CHANGED,
    LAST_SIGNAL
};

typedef struct 
{
    TapiocaUIBuddyData *buddy;
    GdkPixbuf *img_call_active;
    GdkPixbuf *img_call_normal;
} TapiocaUIBuddyStore;
    
struct _TapiocaUIBuddyListPrivate
{
    TapiocaUIDB *db;
    GtkListStore *buddy_store;
    TapiocaUIBuddyStore *expanded_buddy;
    GtkTreeViewColumn *column_name;
    gint old_width;
};

static void       tapiocaui_buddy_list_class_init               (TapiocaUIBuddyListClass *k);
static void       tapiocaui_buddy_list_init                     (TapiocaUIBuddyList *self);
static void       tapiocaui_buddy_list_reload_db                (TapiocaUIBuddyList *sefl);
static GObject   *tapiocaui_buddy_list_constructor              (GType type, 
                                                                 guint n_construct_properties,
                                                                 GObjectConstructParam *construct_properties);
static void       tapiocaui_buddy_list_finalize                 (GObject *object);

static gchar     *tapiocaui_buddy_list_get_description          (TapiocaUIBuddyData *buddy);
static void       tapiocaui_buddy_list_iter_selected_cb         (TapiocaUIList *self, 
                                                                 GtkTreeIter *iter, 
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_iter_unselected_cb       (TapiocaUIList *self, 
                                                                 GtkTreeIter *iter, 
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_iter_activated_cb        (TapiocaUIList *self, 
                                                                 GtkTreeIter *iter, 
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_db_changed_cb            (TapiocaUIDB *db,
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_db_buddy_added_cb        (TapiocaUIDB *db,
                                                                 gpointer buddy,
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_db_buddy_removed_cb      (TapiocaUIDB *db,
                                                                 gpointer buddy,
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_img_call_activate_cb     (TapiocaUICellRendererPixbuf *cell,
                                                                 const gchar *path,
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_render_buddy_description (GtkTreeViewColumn *col,
                                                                 GtkCellRenderer *renderer,
                                                                 GtkTreeModel *model,
                                                                 GtkTreeIter *iter,
                                                                 gboolean mouse_over,
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_render_pixbuf_call       (GtkTreeViewColumn *col,
                                                                 GtkCellRenderer *renderer,
                                                                 GtkTreeModel *model,
                                                                 GtkTreeIter *iter,
                                                                 gboolean mouse_over,
                                                                 gpointer user_data);
static void       tapiocaui_buddy_list_resize_cb                (GtkWidget *widget,
                                                                 GtkAllocation *allocation,
                                                                 gpointer user_data);
static gboolean   tapiocaui_buddy_list_harvest_buddies          (GtkTreeModel *model,
                                                                 GtkTreePath *path,
                                                                 GtkTreeIter *iter,
                                                                 void **data);
static gint       tapiocaui_buddy_list_sort_iter_compare_func   (GtkTreeModel *model,
                                                                 GtkTreeIter  *a,
                                                                 GtkTreeIter  *b,
                                                                 gpointer      userdata);

static GObjectClass *parent_class = NULL;
static guint tapiocaui_buddy_list_signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE (TapiocaUIBuddyList, tapiocaui_buddy_list, TAPIOCAUI_TYPE_LIST)

static void     
tapiocaui_buddy_list_class_init (TapiocaUIBuddyListClass *k)
{
    GObjectClass *object_class = G_OBJECT_CLASS (k);
    parent_class = g_type_class_peek_parent (k);
    object_class->constructor = tapiocaui_buddy_list_constructor;
    object_class->finalize = tapiocaui_buddy_list_finalize;
    
    tapiocaui_buddy_list_signals[BUDDY_ACTIVATED] =
        g_signal_new ("buddy-activated",
                      G_OBJECT_CLASS_TYPE (object_class),                          
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIBuddyListClass, buddy_activated),
                      NULL, NULL,
                      tapiocaui_marshal_VOID__POINTER_BOOLEAN,
                      G_TYPE_NONE,
                      2,
                      G_TYPE_POINTER,
                      G_TYPE_BOOLEAN);
    tapiocaui_buddy_list_signals[BUDDY_SELECTION_CHANGED] =
        g_signal_new ("buddy-selection-changed",
                      G_OBJECT_CLASS_TYPE (object_class),                          
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIBuddyListClass, buddy_selection_changed),
                      NULL, NULL,
                      tapiocaui_marshal_VOID__POINTER_BOOLEAN,
                      G_TYPE_NONE,
                      2,
                      G_TYPE_POINTER,
                      G_TYPE_BOOLEAN);
}

static void
tapiocaui_buddy_list_init (TapiocaUIBuddyList *self)
{    
    self->priv = g_new0 (TapiocaUIBuddyListPrivate, 1);    
}

static GObject *
tapiocaui_buddy_list_constructor (GType type, 
                                  guint n_construct_properties,
                                  GObjectConstructParam *construct_properties)
{
    TapiocaUIBuddyList *self;
    TapiocaUIBuddyListClass *klass;
    GObjectClass *parent_class;
    GtkTreeViewColumn *col;
    GtkCellRenderer *render;
    GtkTreeSortable *sortable;
    GtkSortType order = GTK_SORT_ASCENDING;
    
    klass = TAPIOCAUI_BUDDY_LIST_CLASS (g_type_class_peek (TAPIOCAUI_TYPE_BUDDY_LIST));

    parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
    self = TAPIOCAUI_BUDDY_LIST (parent_class->constructor (type, n_construct_properties,
                                                            construct_properties));
         
    self->priv->buddy_store =  gtk_list_store_new (1, G_TYPE_POINTER);
    gtk_tree_selection_set_mode (tapiocaui_list_get_selection (TAPIOCAUI_LIST (self)),
                                 GTK_SELECTION_MULTIPLE);

    sortable = GTK_TREE_SORTABLE (self->priv->buddy_store);
    gtk_tree_sortable_set_sort_func (sortable, 
                                     0, 
                                     tapiocaui_buddy_list_sort_iter_compare_func,
                                     NULL, 
                                     NULL);
    gtk_tree_sortable_set_sort_column_id (sortable, 0, order);

                                                        
    // BUDDY-NAME-COLUMN
    col = gtk_tree_view_column_new ();
    render = gtk_cell_renderer_text_new ();
    gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_pack_start (col, render, TRUE);
    gtk_tree_view_column_set_resizable (col, FALSE);
    tapiocaui_list_append_collumn (TAPIOCAUI_LIST(self), col,
                                   render,
                                   tapiocaui_buddy_list_render_buddy_description,
                                   (gpointer) self);
    self->priv->column_name = col;                                    
                                                                        
    // BUDDY-CALL-COLUMN
    col = gtk_tree_view_column_new ();
    render = tapiocaui_cell_renderer_pixbuf_new ();
    gtk_tree_view_column_set_resizable (col, FALSE);
    gtk_tree_view_column_pack_start (col, render, TRUE);    
    g_object_set (render, "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, 
                  "xalign", 1.0, NULL);    
    tapiocaui_list_append_collumn (TAPIOCAUI_LIST(self), col,
                                   render,
                                   tapiocaui_buddy_list_render_pixbuf_call,
                                   (gpointer) self);
    
    g_signal_connect_object (G_OBJECT (render),
                             "activate",
                             G_CALLBACK (tapiocaui_buddy_list_img_call_activate_cb),
                             self,
                             0);                     

    // set model
    tapiocaui_list_set_model (TAPIOCAUI_LIST(self), 
                              GTK_TREE_MODEL (self->priv->buddy_store));    

    g_signal_connect_object (G_OBJECT (self),
                             "iter-selected",
                             G_CALLBACK (tapiocaui_buddy_list_iter_selected_cb),
                             self,
                             0);
                         
    g_signal_connect_object (G_OBJECT (self),
                             "iter-unselected",
                             G_CALLBACK (tapiocaui_buddy_list_iter_unselected_cb),
                             self,
                             0);

    g_signal_connect_object (G_OBJECT (self),
                             "iter-activated",
                             G_CALLBACK (tapiocaui_buddy_list_iter_activated_cb),
                             self,
                             0);
                     
    g_signal_connect_object (G_OBJECT (self),
                             "size-allocate",
                             G_CALLBACK (tapiocaui_buddy_list_resize_cb),
                             self,
                             0);                             
    return G_OBJECT (self);
}

static void
tapiocaui_buddy_list_finalize (GObject *object)
{
    TapiocaUIBuddyList *self = TAPIOCAUI_BUDDY_LIST (object);
    self->priv->db = NULL;    
    g_free (self->priv);
    
    G_OBJECT_CLASS (parent_class)->finalize (object);
}

TapiocaUIBuddyList * 
tapiocaui_buddy_list_new (void)
{
    TapiocaUIBuddyList *self;
    self = g_object_new (TAPIOCAUI_TYPE_BUDDY_LIST, 
                         "hadjustment", NULL,
                         "vadjustment", NULL,
                         "hscrollbar_policy", GTK_POLICY_AUTOMATIC,
                         "vscrollbar_policy", GTK_POLICY_AUTOMATIC,
                         "shadow_type", GTK_SHADOW_IN,
                         NULL);
    return self;
}

void
tapiocaui_buddy_list_connect_db (TapiocaUIBuddyList *self,
                                 TapiocaUIDB *db)
{
    GList *lst_db;
    
    g_return_if_fail (self->priv->db == NULL);
  
    tapiocaui_buddy_list_clear (self);
    lst_db = tapiocaui_db_get_buddy_list (db, TAPIOCAUI_DB_ENTRY_BUDDY);    
    tapiocaui_buddy_list_append_list (self, lst_db);    
    self->priv->db = db;
    g_signal_connect (G_OBJECT (db), 
                      "db-changed",
                      G_CALLBACK (tapiocaui_buddy_list_db_changed_cb),
                      self);
    g_signal_connect (G_OBJECT (db), 
                      "db-buddy-added",
                      G_CALLBACK (tapiocaui_buddy_list_db_buddy_added_cb),
                      self);
    g_signal_connect (G_OBJECT (db), 
                      "db-buddy-removed",
                      G_CALLBACK (tapiocaui_buddy_list_db_buddy_removed_cb),
                      self);
}

void
tapiocaui_buddy_list_disconnect_db (TapiocaUIBuddyList *self)
{
    self->priv->db = NULL;
}

void
tapiocaui_buddy_list_clear (TapiocaUIBuddyList *self)
{
    gtk_list_store_clear (self->priv->buddy_store);
}

void
tapiocaui_buddy_list_append_buddy (TapiocaUIBuddyList *self,
                                   TapiocaUIBuddyData *buddy)
{
    GError *error = NULL;
    TapiocaUIBuddyStore *store;
    GtkTreeIter iter;
    
    g_return_if_fail (self->priv->db == NULL);
    
    store = g_new0 (TapiocaUIBuddyStore, 1);
    store->buddy = buddy;
    store->img_call_active = gdk_pixbuf_new_from_file_at_size (
            tapiocaui_file ("call_over.png"), 21, 24, &error);
    
    gtk_list_store_append (self->priv->buddy_store,&iter);

    gtk_list_store_set (self->priv->buddy_store, 
                        &iter,
                        0, store, -1);
}

void 
tapiocaui_buddy_list_append_list (TapiocaUIBuddyList *self,
                                  GList *lst_buddy)
{
    GList *lst = lst_buddy;
    TapiocaUIBuddyData *buddy = NULL;
    
    g_return_if_fail (self->priv->db == NULL);
    
    while (lst) {
        buddy = (TapiocaUIBuddyData *) (lst->data);
        tapiocaui_buddy_list_append_buddy (self, buddy);
        lst = lst->next;        
    }
}

void
tapiocaui_buddy_list_remove_buddy (TapiocaUIBuddyList  *self,
                                   TapiocaUIBuddyData  *buddy)
{
    TapiocaUIBuddyStore *store = NULL;
    GtkTreeIter iter;
    gboolean valid;

    g_return_if_fail (self->priv->db == NULL);
    
    valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->buddy_store), &iter);

    while (valid) {
        gtk_tree_model_get (GTK_TREE_MODEL (self->priv->buddy_store), &iter, 0, &store, -1);
        if (store) {
            if ((g_strcasecmp (store->buddy->id, buddy->id) == 0) &&
                (g_strcasecmp (store->buddy->group, buddy->group) == 0)) {
                gtk_list_store_remove (self->priv->buddy_store, &iter);
                break;
            }
        }
        valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->buddy_store), &iter);
    }
}

static void
tapiocaui_buddy_list_render_buddy_description (GtkTreeViewColumn *col,
                                               GtkCellRenderer *renderer,
                                               GtkTreeModel *model,
                                               GtkTreeIter *iter,
                                               gboolean mouse_over,
                                               gpointer user_data)
{
    TapiocaUIBuddyStore *store;
    gchar *desc;
    
    gtk_tree_model_get (model, iter, 0, &store, -1);    
    if(store)
    {
      desc = tapiocaui_buddy_list_get_description (store->buddy); 
      g_object_set (renderer, "markup", desc, NULL); 
      g_free (desc);
    }
} 

static void
tapiocaui_buddy_list_render_pixbuf_call (GtkTreeViewColumn *col,
                                         GtkCellRenderer *renderer,
                                         GtkTreeModel *model,
                                         GtkTreeIter *iter,
                                         gboolean mouse_over,
                                         gpointer user_data)
{
    TapiocaUIBuddyPresence *tmp;
    TapiocaUIBuddyStore *store;
    gboolean sensitive = FALSE;

    gtk_tree_model_get (model, iter, 0, &store, -1);

    if(store) 
        if (g_hash_table_size (store->buddy->jids) != 0) {
            tmp = g_hash_table_find (store->buddy->jids, (GHRFunc)tapiocaui_db_voice_cap_hash_filter, NULL);
            if ((tmp) && (tmp->voice_cap))
                sensitive = (mouse_over);
        }

    g_object_set (renderer, "pixbuf", store->img_call_active,
                  "sensitive", sensitive, NULL);    
}

static gchar *
tapiocaui_buddy_list_get_description (TapiocaUIBuddyData *buddy) 
{
    TapiocaUIBuddyPresence *tmp;

    g_return_val_if_fail (buddy != NULL, NULL);
        
    if (g_hash_table_size (buddy->jids) == 0) {
        if ((buddy->subscription_state == TAPIOCA_CONTACT_LIST_SUBSCRIPTION_STATE_NONE) ||
            (buddy->subscription_state == TAPIOCA_CONTACT_LIST_SUBSCRIPTION_STATE_TO))
            return g_strdup_printf ("%s\n<i>Pending</i>", buddy->name);
        else
            return g_strdup_printf ("%s\nOffline", buddy->name);
    }
    else {
        tmp = g_hash_table_find (buddy->jids, (GHRFunc)gtk_true, NULL);
        return g_strdup_printf ("<b>%s</b>\n%s", buddy->name, tmp->status_desc);
    }
}

static void 
tapiocaui_buddy_list_iter_selected_cb (TapiocaUIList *self, 
                                       GtkTreeIter *iter, 
                                       gpointer user_data)
{
    TapiocaUIBuddyStore *store;

    gtk_tree_model_get (GTK_TREE_MODEL (TAPIOCAUI_BUDDY_LIST (self)->priv->buddy_store), 
                        iter, 0, &store, -1);
    if(store)
    {
      g_signal_emit (G_OBJECT (self), 
                     tapiocaui_buddy_list_signals [BUDDY_SELECTION_CHANGED], 0, 
                     store->buddy, TRUE);
    }
}

static void 
tapiocaui_buddy_list_iter_unselected_cb (TapiocaUIList *self, 
                                         GtkTreeIter *iter, 
                                         gpointer user_data)
{
    TapiocaUIBuddyStore *store;

    gtk_tree_model_get (GTK_TREE_MODEL (TAPIOCAUI_BUDDY_LIST (self)->priv->buddy_store), 
                        iter, 0, &store, -1);
    g_signal_emit (G_OBJECT (self), 
                   tapiocaui_buddy_list_signals [BUDDY_SELECTION_CHANGED], 0, 
                   store->buddy, FALSE);
}

void
tapiocaui_buddy_list_set_filter (TapiocaUIBuddyList *self,
                                 const gchar *value)
{
    if (self->priv->db != NULL) {
        if (value == NULL) {
            TapiocaUIDB *db = self->priv->db;
            tapiocaui_buddy_list_disconnect_db (self);
            tapiocaui_buddy_list_clear (self);
            tapiocaui_buddy_list_connect_db (self, db);            
        } else {
            GList *filter;
            TapiocaUIDB *db = self->priv->db;        
            self->priv->db = NULL;
            filter = tapiocaui_db_filter (db, 
                                          TAPIOCAUI_DB_ENTRY_BUDDY, 
                                          TAPIOCAUI_DB_FIELD_NAME, 
                                          value);
            tapiocaui_buddy_list_clear (self);
            tapiocaui_buddy_list_append_list (self, filter);
            self->priv->db = db;
        }        
    } else {
        // TODO
    }    
}

static void
tapiocaui_buddy_list_db_buddy_added_cb (TapiocaUIDB *db,
                                        gpointer buddy,
                                        gpointer user_data)
{
    TapiocaUIBuddyList *self = TAPIOCAUI_BUDDY_LIST (user_data);
    TapiocaUIDB *_db = self->priv->db;

    self->priv->db = NULL;
    
    tapiocaui_buddy_list_append_buddy (self, buddy); 

    self->priv->db = _db;
}

static void
tapiocaui_buddy_list_db_buddy_removed_cb (TapiocaUIDB *db,
                                          gpointer buddy,
                                          gpointer user_data)
{
    TapiocaUIBuddyList *self = TAPIOCAUI_BUDDY_LIST (user_data);
    TapiocaUIDB *_db = self->priv->db;

    self->priv->db = NULL;
    
    tapiocaui_buddy_list_remove_buddy (self, buddy); 

    self->priv->db = _db;
}

static void
tapiocaui_buddy_list_db_changed_cb (TapiocaUIDB *db,
                                    gpointer user_data)
{
    tapiocaui_buddy_list_reload_db (TAPIOCAUI_BUDDY_LIST (user_data));    
}

static void
tapiocaui_buddy_list_reload_db (TapiocaUIBuddyList *self)
{
    TapiocaUIDB *db = self->priv->db;
    self->priv->db = NULL;
    tapiocaui_buddy_list_clear (self);
    tapiocaui_buddy_list_append_list (self, 
                                      tapiocaui_db_get_buddy_list (db, TAPIOCAUI_DB_ENTRY_BUDDY));
    self->priv->db = db;
}

TapiocaUIBuddyData *
tapiocaui_buddy_list_get_buddy (TapiocaUIBuddyList *self,
                                GtkTreeIter *iter)
{
    TapiocaUIBuddyStore *store;
    gtk_tree_model_get (GTK_TREE_MODEL (TAPIOCAUI_BUDDY_LIST (self)->priv->buddy_store), 
                        iter, 0, &store, -1);
    
    return store->buddy;
}

GList *
tapiocaui_buddy_get_selected_buddies (TapiocaUIBuddyList *self)
{
    GList *list = NULL;
    GtkTreeSelection *selection = tapiocaui_list_get_selection (TAPIOCAUI_LIST (self));
    gtk_tree_selection_selected_foreach (selection,
                                         (GtkTreeSelectionForeachFunc) tapiocaui_buddy_list_harvest_buddies,
                                         (gpointer) &list);
    return list;
}

static void 
tapiocaui_buddy_list_iter_activated_cb (TapiocaUIList *self, 
                                        GtkTreeIter *iter, 
                                        gpointer user_data)
{
    TapiocaUIBuddyStore *store;

    gtk_tree_model_get (GTK_TREE_MODEL (TAPIOCAUI_BUDDY_LIST (self)->priv->buddy_store), 
                                        iter, 0, &store, -1);
    g_signal_emit (G_OBJECT (self), 
                   tapiocaui_buddy_list_signals [BUDDY_ACTIVATED], 0, 
                   store->buddy, FALSE);
}

static void     
tapiocaui_buddy_list_img_call_activate_cb (TapiocaUICellRendererPixbuf *cell,
                                           const gchar *path_str,
                                           gpointer user_data)
{
    GtkTreeIter iter;
    GtkTreePath *path;
    TapiocaUIBuddyPresence *tmp;
    TapiocaUIBuddyStore *store;
    TapiocaUIBuddyList *self = TAPIOCAUI_BUDDY_LIST (user_data);
    
    path = gtk_tree_path_new_from_string (path_str);
    
    gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->buddy_store), 
                             &iter, path);
    gtk_tree_model_get (GTK_TREE_MODEL (self->priv->buddy_store), &iter, 0, &store, -1);
    tmp = g_hash_table_find (store->buddy->jids, (GHRFunc)tapiocaui_db_voice_cap_hash_filter, NULL);

    if ((tmp) && (tmp->voice_cap))
        g_signal_emit (G_OBJECT (self), tapiocaui_buddy_list_signals [BUDDY_ACTIVATED], 0, store->buddy, TRUE);

    gtk_tree_path_free  (path);
}

static void
tapiocaui_buddy_list_resize_cb (GtkWidget *widget,
                                GtkAllocation *allocation,
                                gpointer user_data)
{
    TapiocaUIBuddyList *self = TAPIOCAUI_BUDDY_LIST (user_data);
    if (self->priv->old_width != allocation->width) {
        gtk_tree_view_column_set_fixed_width (self->priv->column_name,
                                              allocation->width - 50);
        self->priv->old_width = allocation->width;
    }
}

static gboolean
tapiocaui_buddy_list_harvest_buddies (GtkTreeModel *model,
                                      GtkTreePath *path,
                                      GtkTreeIter *iter,
                                      void **data)
{
    GList **list = (GList **) data;
    TapiocaUIBuddyStore *buddy;

    gtk_tree_model_get (model, iter, 0, &buddy, -1);

    *list = g_list_append (*list, buddy->buddy);

    return FALSE;
}

void
tapiocaui_buddy_list_update (TapiocaUIBuddyList *self)
{
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self->priv->buddy_store), 0, GTK_SORT_DESCENDING);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self->priv->buddy_store), 0, GTK_SORT_ASCENDING);
}

static gint
tapiocaui_buddy_list_sort_iter_compare_func (GtkTreeModel *model,
                                             GtkTreeIter  *a,
                                             GtkTreeIter  *b,
                                             gpointer      userdata)
{
    TapiocaUIBuddyStore *store_a = NULL;
    TapiocaUIBuddyStore *store_b = NULL;
    guint status_a = 100;
    guint status_b = 100;

    gtk_tree_model_get (model, a, 0, &store_a, -1);
    gtk_tree_model_get (model, b, 0, &store_b, -1);

    if (store_a && store_b) {
        if (g_hash_table_size (store_a->buddy->jids) != 0) 
            status_a = ((TapiocaUIBuddyPresence*) g_hash_table_find (store_a->buddy->jids, (GHRFunc)gtk_true, NULL))->priority;
        
        if (g_hash_table_size (store_b->buddy->jids) != 0) 
            status_b = ((TapiocaUIBuddyPresence*) g_hash_table_find (store_b->buddy->jids, (GHRFunc)gtk_true, NULL))->priority;
    }
    
    if (status_a == status_b) {
        return g_ascii_strncasecmp (store_a->buddy->name, store_b->buddy->name, strlen (store_a->buddy->name));
    }
    return ((status_a < status_b) ? -1 : 1);
}
