/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glade/glade.h>
#include <glib/gi18n.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkrange.h>
#include <gtk/gtklabel.h>
#include <gtk/gtktextview.h>
#include <gtk/gtkimage.h>
#include <gtk/gtkscrolledwindow.h>
#include <time.h>

#include "tapiocaui_communication.h"
#include "tapiocaui_stock_icons.h"
#include "tapiocaui_utils.h"

#define VOLUME_MIN 0
#define VOLUME_MAX 100

enum
{
    VOLUME_CHANGE,
    INPUT_SOUND_MUTE,
    INPUT_SOUND_UNMUTE,
    SEND_MESSAGE,
    CALL_ANSWER,
    CALL_REJECT,
    CALL_MAKE,
    CALL_HOLD,
    CALL_UNHOLD,
    CALL_DISCONNECT,
    QUIT,
    LAST_SIGNAL
};

enum
{
    PROP_0,
    PROP_USER,
    PROP_BUDDY
};

struct _TapiocaUICommunicationPrivate
{
    TapiocaUIBuddyData *buddy;
    GtkWidget *lbl_time;
    GtkWidget *lbl_status;
    GtkWidget *evnt_output_volume;

    GtkWidget *img_output_sound;
    GtkWidget *img_input_sound;

    GtkWidget *input_message;
    GtkWidget *btn_output_sound;
    GtkWidget *btn_input_sound;
    GtkWidget *txt_msglist;

    GtkWidget *evnt_action;
    GtkWidget *img_action;
    GtkWidget *lbl_action;

    GtkWidget *evnt_action2;
    GtkWidget *img_action2;
    GtkWidget *lbl_action2;
    GtkWidget *img_volume[5];

    GdkPixbuf *img_call_hold;
    GdkPixbuf *img_call_unhold;
    GdkPixbuf *img_call_make;
    GdkPixbuf *img_call_end;
    GdkPixbuf *img_volume_off;
    GdkPixbuf *img_volume_on;
    
    GTimer *timer;
    TapiocaUICommunicationStatus status;
    guint timer_func;
    guint volume;
    guint volume_mute;
    gchar *message_own;
    gchar *user;
    GtkTextMark *title_mark_begin;
    GtkTextMark *title_mark_end;
};

static void     tapiocaui_communication_class_init      (TapiocaUICommunicationClass *k);
static void     tapiocaui_communication_init            (TapiocaUICommunication *self);
static void     tapiocaui_communication_finalize        (GObject *object);
static GObject *tapiocaui_communication_constructor     (GType type,
                                                         guint n_construct_properties,
                                                         GObjectConstructParam *construct_properties);
static void     tapiocaui_communication_set_property    (GObject *object,
                                                         guint prop_id,
                                                         const GValue *value,
                                                         GParamSpec *pspec);
static void     tapiocaui_communication_get_property    (GObject *object,
                                                         guint prop_id,
                                                         GValue *value,
                                                         GParamSpec *pspec);
static void     tapiocaui_communication_start_time      (TapiocaUICommunication *self);
static void     tapiocaui_communication_stop_time       (TapiocaUICommunication *self);
static void     tapiocaui_communication_destroy_cb      (GtkWidget *widget,
                                                         gpointer user_data);
static gboolean tapiocaui_communication_action_cb       (GtkWidget *widget,
                                                         GdkEventButton *event,
                                                         gpointer user_data);
static gboolean tapiocaui_communication_action2_cb      (GtkWidget *widget,
                                                         GdkEventButton *event,
                                                         gpointer user_data);
static void     tapiocaui_communication_send_message_cb (gpointer control,
                                                         gpointer user_data);
static void     tapiocaui_communication_output_mute_cb  (GtkButton *button,
                                                         gpointer user_data);
static void     tapiocaui_communication_input_mute_cb   (GtkButton *button,
                                                         gpointer user_data);
static gboolean tapiocaui_communication_timer_cb        (gpointer data);
static gboolean tapiocaui_communication_volume_cb       (GtkWidget *widget,
                                                         GdkEventButton *event,
                                                         gpointer user_data);
static gboolean tapiocaui_communication_auto_scroll     (gpointer data);

static guint tapiocaui_communication_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

G_DEFINE_TYPE (TapiocaUICommunication, tapiocaui_communication, GTK_TYPE_WINDOW)

static void
tapiocaui_communication_class_init (TapiocaUICommunicationClass *k)
{
    GObjectClass *object_class = G_OBJECT_CLASS (k);
    parent_class = g_type_class_peek_parent (k);

    object_class->finalize = tapiocaui_communication_finalize;
    object_class->set_property = tapiocaui_communication_set_property;
    object_class->get_property = tapiocaui_communication_get_property;
    object_class->constructor = tapiocaui_communication_constructor;

    g_object_class_install_property (object_class,
                                     PROP_BUDDY,
                                     g_param_spec_pointer ("buddy",
                                                           "buddy",
                                                           "Buddy",
                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));

    g_object_class_install_property (object_class,
                                     PROP_USER,
                                     g_param_spec_string ("user",
                                                          "user",
                                                          "user",
                                                          "",
                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
    tapiocaui_communication_signals[VOLUME_CHANGE] =
        g_signal_new ("volume-change",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, volume_change),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__UINT,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_UINT);
    tapiocaui_communication_signals[INPUT_SOUND_MUTE] =
        g_signal_new ("input-sound-mute",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, input_sound_mute),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
    tapiocaui_communication_signals[INPUT_SOUND_UNMUTE] =
        g_signal_new ("input-sound-unmute",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, input_sound_unmute),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
    tapiocaui_communication_signals[SEND_MESSAGE] =
        g_signal_new ("send-message",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, send_message),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
    tapiocaui_communication_signals[CALL_ANSWER] =
        g_signal_new ("call-answer",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, call_answer),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
    tapiocaui_communication_signals[CALL_HOLD] =
        g_signal_new ("call-hold",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, call_hold),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);                      
    tapiocaui_communication_signals[CALL_UNHOLD] =
        g_signal_new ("call-unhold",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, call_unhold),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);                                            
    tapiocaui_communication_signals[CALL_REJECT] =
        g_signal_new ("call-reject",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, call_reject),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
    tapiocaui_communication_signals[CALL_DISCONNECT] =
        g_signal_new ("call-disconnect",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, call_disconect),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);                      
    tapiocaui_communication_signals[CALL_MAKE] =
        g_signal_new ("call-make",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, call_make),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);                      
    tapiocaui_communication_signals[QUIT] =
        g_signal_new ("quit",
                      TAPIOCAUI_TYPE_COMMUNICATION,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUICommunicationClass, quit),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
}

static void
tapiocaui_communication_init (TapiocaUICommunication *self)
{
    self->priv = g_new0 (TapiocaUICommunicationPrivate, 1);
    self->priv->timer_func = -1;
}

static GObject *
tapiocaui_communication_constructor (GType type,
                                     guint n_construct_properties,
                                     GObjectConstructParam *construct_properties)
{
    GladeXML *xml;
    TapiocaUICommunicationClass *klass;
    TapiocaUICommunication *self;
    GtkWidget *tmp;
    GError *error=NULL;
    GtkTextBuffer *buffer;
    gchar *user_name;
    GdkColor color;

    klass = TAPIOCAUI_COMMUNICATION_CLASS (g_type_class_peek (TAPIOCAUI_TYPE_COMMUNICATION));
    parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
    self = TAPIOCAUI_COMMUNICATION (parent_class->constructor (type, n_construct_properties,
                                                               construct_properties));

    gtk_window_set_title (GTK_WINDOW (self), self->priv->buddy->name);
    xml = glade_xml_new (tapiocaui_file ("communication-window.glade"), "box", NULL);
    glade_xml_signal_autoconnect (xml);

    gtk_container_add (GTK_CONTAINER (self),
                       glade_xml_get_widget (xml, "box"));

    tmp = glade_xml_get_widget (xml, "lbl_user");
    user_name  = g_strdup_printf ("<b><big>%s</big></b>", self->priv->buddy->name); 
    gtk_label_set_markup (GTK_LABEL (tmp), user_name);
    g_free (user_name);
    self->priv->btn_output_sound = glade_xml_get_widget (xml, "btn_output_sound");
    g_signal_connect (G_OBJECT (self->priv->btn_output_sound),
                      "clicked",
                      G_CALLBACK (tapiocaui_communication_output_mute_cb),
                      self);
    self->priv->btn_input_sound = glade_xml_get_widget (xml, "btn_input_sound");
    g_signal_connect (G_OBJECT (self->priv->btn_input_sound),
                      "clicked",
                      G_CALLBACK (tapiocaui_communication_input_mute_cb),
                      self);

    self->priv->img_output_sound = glade_xml_get_widget (xml, "img_output_sound");
    gtk_image_set_from_file (GTK_IMAGE (self->priv->img_output_sound), 
                             tapiocaui_file ("icon_sound.png"));
    self->priv->img_input_sound = glade_xml_get_widget (xml, "img_input_sound");
    gtk_image_set_from_file (GTK_IMAGE (self->priv->img_input_sound), 
                             tapiocaui_file ("icon_mic.png"));
    self->priv->input_message = glade_xml_get_widget (xml, "entry_message");
    g_signal_connect (G_OBJECT (self->priv->input_message),
                      "activate",
                      G_CALLBACK (tapiocaui_communication_send_message_cb),
                      self);
    tmp = glade_xml_get_widget (xml, "btn_send");
    g_signal_connect (G_OBJECT (tmp),
                      "clicked",
                      G_CALLBACK (tapiocaui_communication_send_message_cb),
                      self);
    
    self->priv->img_action = glade_xml_get_widget (xml, "img_action");
    self->priv->lbl_action = glade_xml_get_widget (xml, "lbl_action");
    self->priv->evnt_action = glade_xml_get_widget (xml, "evnt_action");
    gtk_widget_add_events(self->priv->evnt_action, GDK_BUTTON_RELEASE_MASK);
    gtk_widget_add_events(self->priv->evnt_action, GDK_BUTTON_PRESS_MASK);
    gtk_widget_add_events(self->priv->evnt_action, GDK_BUTTON1_MOTION_MASK);
    g_signal_connect (G_OBJECT (self->priv->evnt_action),
                      "button-press-event",
                      G_CALLBACK (tapiocaui_communication_action_cb),
                      self);

    self->priv->img_action2 = glade_xml_get_widget (xml, "img_action2");
    self->priv->lbl_action2 = glade_xml_get_widget (xml, "lbl_action2");
    self->priv->evnt_action2 = glade_xml_get_widget (xml, "evnt_action2");
    gtk_widget_add_events(self->priv->evnt_action2, GDK_BUTTON_RELEASE_MASK);
    gtk_widget_add_events(self->priv->evnt_action2, GDK_BUTTON_PRESS_MASK);
    gtk_widget_add_events(self->priv->evnt_action2, GDK_BUTTON1_MOTION_MASK);    
    g_signal_connect (G_OBJECT (self->priv->evnt_action2),
                      "button-press-event",
                      G_CALLBACK (tapiocaui_communication_action2_cb),
                      self);

    g_signal_connect (G_OBJECT (self),
                      "destroy",
                      G_CALLBACK (tapiocaui_communication_destroy_cb),
                      self);


    self->priv->img_volume[0] = glade_xml_get_widget (xml, "img_vol1");
    self->priv->img_volume[1] = glade_xml_get_widget (xml, "img_vol2");
    self->priv->img_volume[2] = glade_xml_get_widget (xml, "img_vol3");
    self->priv->img_volume[3] = glade_xml_get_widget (xml, "img_vol4");
    self->priv->img_volume[4] = glade_xml_get_widget (xml, "img_vol5");
    self->priv->evnt_output_volume = glade_xml_get_widget (xml, "evnt_vol");
    gtk_widget_add_events(self->priv->evnt_output_volume, GDK_BUTTON_RELEASE_MASK);
    gtk_widget_add_events(self->priv->evnt_output_volume, GDK_BUTTON_PRESS_MASK);
    gtk_widget_add_events(self->priv->evnt_output_volume, GDK_BUTTON1_MOTION_MASK);    
    g_signal_connect (G_OBJECT (self->priv->evnt_output_volume),
                      "button-press-event",
                      G_CALLBACK (tapiocaui_communication_volume_cb),
                      self);

    self->priv->lbl_status = glade_xml_get_widget (xml, "lbl_status");
    self->priv->lbl_time = glade_xml_get_widget (xml, "lbl_time");
    //setup text message
    self->priv->txt_msglist = glade_xml_get_widget (xml, "txt_msglist");
    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->priv->txt_msglist));
    color.red = 34952;
    color.green = 34952;
    color.blue = 34952;    
    gtk_text_buffer_create_tag (buffer, "bold", "weight", PANGO_WEIGHT_BOLD, NULL);
    gtk_text_buffer_create_tag (buffer, "fg_gray", "foreground_gdk", &color, 
                                                   "size", 7 * PANGO_SCALE, NULL);
    color.red = 24415;
    color.green = 47545; 
    color.blue = 24415;
    gtk_text_buffer_create_tag (buffer, "fg_green", "foreground_gdk", &color, NULL);

    // load images
    self->priv->img_call_make = gdk_pixbuf_new_from_file_at_size (tapiocaui_file ("call.png"), 
                                                                  31, 39, &error);                                                                  
    g_assert (error == NULL);                                                                  
    self->priv->img_call_end = gdk_pixbuf_new_from_file_at_size (tapiocaui_file ("call_end.png"), 
                                                                 31, 39, &error);
    g_assert (error == NULL);
    self->priv->img_call_hold = gdk_pixbuf_new_from_file_at_size (tapiocaui_file ("call_hold.png"), 
                                                                  31, 39, &error);
    g_assert (error == NULL);                                                                    
    self->priv->img_call_unhold = gdk_pixbuf_new_from_file_at_size (tapiocaui_file ("call_unhold.png"), 
                                                                  31, 39, &error);
    g_assert (error == NULL);                                                                        
    self->priv->img_volume_on = gdk_pixbuf_new_from_file (tapiocaui_file ("volume_on.png"), &error);
    g_assert (error == NULL);
    self->priv->img_volume_off = gdk_pixbuf_new_from_file (tapiocaui_file ("volume_off.png"), &error);
    g_assert (error == NULL);                                                                        
    
    self->priv->timer = g_timer_new ();
    tapiocaui_communication_set_volume (self, 0);
    tapiocaui_communication_set_status (self, TAPIOCAUI_CALL_IDLE);
    
    gtk_widget_grab_focus (self->priv->input_message);
    
    g_object_unref (G_OBJECT (xml));
    return G_OBJECT (self);
}

static void
tapiocaui_communication_finalize (GObject *object)
{
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (object);
    
    g_return_if_fail (object != NULL);
    g_return_if_fail (TAPIOCAUI_IS_COMMUNICATION (object));
    tapiocaui_communication_stop_time (self);
    tapiocaui_debug ("finalize");
    
    g_object_unref (self->priv->img_call_make);
    g_object_unref (self->priv->img_call_end);
    g_object_unref (self->priv->img_call_hold);
    g_object_unref (self->priv->img_call_unhold);

    self = TAPIOCAUI_COMMUNICATION (object);
    g_timer_destroy (self->priv->timer);
    g_free (self->priv);

    G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
tapiocaui_communication_set_property (GObject *object,
                                      guint prop_id,
                                      const GValue *value,
                                      GParamSpec *pspec)
{
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (object);
  
    switch (prop_id)
    {
        case PROP_USER:
            self->priv->user = g_value_dup_string (value);
            break;
        case PROP_BUDDY:
            self->priv->buddy = g_value_get_pointer (value);
            break;  
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
tapiocaui_communication_get_property (GObject *object,
                                      guint prop_id,
                                      GValue *value,
                                      GParamSpec *pspec)
{
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (object);

    switch (prop_id)
    {
        case PROP_USER:       
            g_value_set_string (value, self->priv->user);
            break;
        case PROP_BUDDY:
            g_value_set_pointer (value, self->priv->buddy);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

GtkWidget* 
tapiocaui_communication_new (const gchar *user, TapiocaUIBuddyData *buddy)
{
    TapiocaUICommunication *self;
    self = g_object_new (TAPIOCAUI_TYPE_COMMUNICATION,
                         "user", user,
                         "buddy", buddy,
                         NULL);

    return GTK_WIDGET (self);
}

static gboolean 
tapiocaui_communication_action_cb (GtkWidget *widget,
                                   GdkEventButton *event,
                                   gpointer user_data)
{
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (user_data);
    TapiocaUIBuddyPresence *tmp;

    tmp = g_hash_table_find (self->priv->buddy->jids, (GHRFunc)tapiocaui_db_voice_cap_hash_filter, NULL);

    if (!((tmp) && (tmp->voice_cap)))
        return FALSE;

    if (event->button != 1) 
        return FALSE;
    
    switch (self->priv->status) 
    {
        case TAPIOCAUI_CALL_HOLD:
            g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [CALL_UNHOLD], 0, self->priv->buddy->id);
            break;                    
        case TAPIOCAUI_CALL_IDLE:
            g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [CALL_MAKE], 0, self->priv->buddy->id);
            break;        
        case TAPIOCAUI_CALL_ANSWERED:      
            g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [CALL_HOLD], 0, self->priv->buddy->id);
            break;
        case TAPIOCAUI_CALL_INCOMING: 
            g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [CALL_REJECT], 0, self->priv->buddy->id);            
            break;        
        case TAPIOCAUI_CALL_RINGING:
            g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [CALL_DISCONNECT], 0, self->priv->buddy->id);            
            break;                                        
    }
    return TRUE;
}
static gboolean
tapiocaui_communication_action2_cb (GtkWidget *widget,
                                    GdkEventButton *event,
                                    gpointer user_data)
{
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (user_data);

    if (event->button != 1) 
        return FALSE;
    
    switch (self->priv->status) 
    {
        case TAPIOCAUI_CALL_ANSWERED:      
        case TAPIOCAUI_CALL_HOLD:
            g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [CALL_DISCONNECT], 0, self->priv->buddy->id);            
            break;
        case TAPIOCAUI_CALL_INCOMING:            
            g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [CALL_ANSWER], 0, self->priv->buddy->id);
            break;            
        case TAPIOCAUI_CALL_RINGING:        
        case TAPIOCAUI_CALL_IDLE:
            break;
    }
    return TRUE;
}

static gboolean
tapiocaui_communication_volume_cb (GtkWidget *widget,
                                   GdkEventButton *event,
                                   gpointer user_data)
{
    tapiocaui_communication_set_volume (TAPIOCAUI_COMMUNICATION (user_data), event->x * 2);
    g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [VOLUME_CHANGE], 0, (guint) event->x * 2);
    return FALSE;
}

static gchar *
tapiocaui_communication_get_time_str (gdouble value)
{
    guint h, m, s;
    gdouble sec = value;

    m = 0;
    h = sec / 3600;
    sec -= h * 3600;
    if (sec >= 60) {
        m = sec / 60;
        sec -= m * 60;
    }
    s = sec;

    return g_strdup_printf ("%02d:%02d:%02d", h, m, s);
}

static gboolean 
tapiocaui_communication_timer_cb (gpointer data)
{
    gulong us;
    gdouble elapsed_time;
    gchar *time_str;
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (data);
    if (self->priv->timer_func == -1) 
        return FALSE;
    
    elapsed_time = g_timer_elapsed (self->priv->timer, &us);
    time_str = tapiocaui_communication_get_time_str (elapsed_time);
    gtk_label_set_text (GTK_LABEL (self->priv->lbl_time),
                        time_str);
    g_free (time_str);
    return TRUE;
}

static void
tapiocaui_communication_start_time (TapiocaUICommunication *self)
{
    tapiocaui_debug ("starting timer");
    g_timer_reset (self->priv->timer);
    g_timer_start (self->priv->timer);
    tapiocaui_debug ("start count");
    self->priv->timer_func = g_timeout_add (1000, tapiocaui_communication_timer_cb, self);
    tapiocaui_communication_timer_cb (self);
}

static void 
tapiocaui_communication_stop_time (TapiocaUICommunication *self)
{
    tapiocaui_debug ("stop count");
    if (self->priv->timer_func != -1) {
        tapiocaui_debug ("time active, finalize");
        g_source_remove  (self->priv->timer_func);
        g_timer_stop (self->priv->timer);
        self->priv->timer_func = -1;
    }
}

void
tapiocaui_communication_set_status (TapiocaUICommunication *self,
                                    TapiocaUICommunicationStatus status)
{
    gchar *status_str = NULL;
    TapiocaUIBuddyPresence *tmp;

    switch (status) 
    {
        case TAPIOCAUI_CALL_IDLE:
            tapiocaui_communication_stop_time (self);            
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_action), 
                                       self->priv->img_call_make);
            
            tmp = g_hash_table_find (self->priv->buddy->jids, (GHRFunc)tapiocaui_db_voice_cap_hash_filter, NULL);
            if (!((tmp) && (tmp->voice_cap)))
                gtk_widget_set_sensitive (self->priv->img_action, FALSE);

            gtk_label_set_text (GTK_LABEL (self->priv->lbl_action), _("call"));
            status_str = g_strdup (self->priv->buddy->id);
            gtk_widget_show (self->priv->lbl_status);
            gtk_widget_hide (self->priv->lbl_time);
            gtk_widget_hide (self->priv->evnt_output_volume);
            gtk_widget_show (self->priv->evnt_action);            
            gtk_widget_show (self->priv->lbl_action);
            gtk_widget_show (self->priv->img_action);        
            gtk_widget_hide (self->priv->evnt_action2);                    
            gtk_widget_hide (self->priv->lbl_action2);
            gtk_widget_hide (self->priv->img_action2);    
            gtk_widget_hide (self->priv->btn_input_sound);
            gtk_widget_hide (self->priv->btn_output_sound);        
            break;
        case TAPIOCAUI_CALL_INCOMING:
            status_str = g_strdup (_("is calling you"));
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_action), 
                                       self->priv->img_call_end);
            gtk_label_set_text (GTK_LABEL (self->priv->lbl_action), _("reject"));                            
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_action2), 
                                       self->priv->img_call_make);
            gtk_label_set_text (GTK_LABEL (self->priv->lbl_action2), _("accept"));
        
            gtk_widget_hide (self->priv->lbl_time);
            gtk_widget_show (self->priv->evnt_output_volume);
            gtk_widget_show (self->priv->evnt_action);            
            gtk_widget_show (self->priv->lbl_action);
            gtk_widget_show (self->priv->img_action);        
            gtk_widget_show (self->priv->evnt_action2);                    
            gtk_widget_show (self->priv->lbl_action2);
            gtk_widget_show (self->priv->img_action2);
            gtk_widget_show (self->priv->btn_input_sound);
            gtk_widget_show (self->priv->btn_output_sound);                
            gtk_widget_set_sensitive (self->priv->btn_input_sound, FALSE);
            gtk_widget_set_sensitive (self->priv->btn_output_sound, FALSE);
            gtk_widget_set_sensitive (self->priv->evnt_output_volume, FALSE);
            break;
        case TAPIOCAUI_CALL_RINGING:
            status_str = g_strdup (_("calling... ring... ring..."));
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_action), 
                                       self->priv->img_call_end);
            gtk_label_set_text (GTK_LABEL (self->priv->lbl_action), _("cancel"));                            
        
            gtk_widget_hide (self->priv->lbl_time);
            gtk_widget_show (self->priv->evnt_output_volume);
            gtk_widget_show (self->priv->evnt_action);            
            gtk_widget_show (self->priv->lbl_action);
            gtk_widget_show (self->priv->img_action);        
            gtk_widget_hide (self->priv->evnt_action2);                    
            gtk_widget_hide (self->priv->lbl_action2);
            gtk_widget_hide (self->priv->img_action2);
            gtk_widget_show (self->priv->btn_input_sound);
            gtk_widget_show (self->priv->btn_output_sound);    
            gtk_widget_set_sensitive (self->priv->btn_input_sound, FALSE);
            gtk_widget_set_sensitive (self->priv->btn_output_sound, FALSE);
            gtk_widget_set_sensitive (self->priv->evnt_output_volume, FALSE);
            break;
        case TAPIOCAUI_CALL_ANSWERED:
            tapiocaui_communication_start_time (self);
            status_str = g_strdup (_("Active Call"));
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_action), 
                                       self->priv->img_call_hold);
            gtk_label_set_text (GTK_LABEL (self->priv->lbl_action), _("hold"));
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_action2), 
                                       self->priv->img_call_end);
            gtk_label_set_text (GTK_LABEL (self->priv->lbl_action2), _("end call"));
                    
            gtk_widget_show (self->priv->lbl_time);
            gtk_widget_show (self->priv->evnt_output_volume);
            gtk_widget_show (self->priv->evnt_action);            
            gtk_widget_show (self->priv->lbl_action);
            gtk_widget_show (self->priv->img_action);        
            gtk_widget_show (self->priv->evnt_action2);                    
            gtk_widget_show (self->priv->lbl_action2);
            gtk_widget_show (self->priv->img_action2);
            gtk_widget_show (self->priv->btn_input_sound);
            gtk_widget_show (self->priv->btn_output_sound);
            gtk_widget_set_sensitive (self->priv->btn_input_sound, TRUE);
            gtk_widget_set_sensitive (self->priv->btn_output_sound, TRUE);
            gtk_widget_set_sensitive (self->priv->evnt_output_volume, TRUE);
            break;        
        case TAPIOCAUI_CALL_HOLD:
            status_str = g_strdup_printf (_("call on hold"));
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_action), 
                                       self->priv->img_call_unhold);
            gtk_label_set_text (GTK_LABEL (self->priv->lbl_action), _("unhold"));
            break;                            
    }
    self->priv->status = status;
    gtk_label_set_text (GTK_LABEL (self->priv->lbl_status), status_str);
    g_free (status_str);
}

void
tapiocaui_communication_set_volume (TapiocaUICommunication *self,
                                    guint volume)
{
    guint index = 0;
    guint image = 0;
    for (image=0; image <= 4; image++) {        
        if (index < volume) { 
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_volume[image]), 
                                       self->priv->img_volume_on);
        } else {
            gtk_image_set_from_pixbuf (GTK_IMAGE (self->priv->img_volume[image]), 
                               self->priv->img_volume_off);
        }
        index+=20;
    }
    self->priv->volume = volume;
}

const TapiocaUIBuddyData *
tapiocaui_communication_get_buddy (TapiocaUICommunication *self)
{
    return self->priv->buddy;
}

guint
tapiocaui_communication_get_volume (TapiocaUICommunication *self)
{
    return self->priv->volume;
}

static void
tapiocaui_communication_destroy_cb (GtkWidget *widget,
                                    gpointer user_data)
{
    g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [QUIT], 0);
}

static void 
tapiocaui_communication_output_mute_cb (GtkButton *button,
                                        gpointer user_data)
{
    gboolean sensitive;
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (user_data);
    g_object_get (G_OBJECT (self->priv->img_output_sound), "sensitive", &sensitive, NULL);
    if (sensitive) {
        gtk_widget_set_sensitive (self->priv->img_output_sound, FALSE);
        gtk_widget_set_sensitive (self->priv->evnt_output_volume, FALSE);
        self->priv->volume_mute = self->priv->volume;
        tapiocaui_communication_set_volume (self, 0);
    } else {
        gtk_widget_set_sensitive (self->priv->img_output_sound, TRUE);
        gtk_widget_set_sensitive (self->priv->evnt_output_volume, TRUE);
        tapiocaui_communication_set_volume (self, self->priv->volume_mute);
    }
    g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [VOLUME_CHANGE], 0, (guint) self->priv->volume);
}

static void 
tapiocaui_communication_input_mute_cb (GtkButton *button,
                                       gpointer user_data)
{
    gboolean sensitive;
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (user_data);
    g_object_get (G_OBJECT (self->priv->img_input_sound), "sensitive", &sensitive, NULL);
    if (sensitive) {        
        gtk_widget_set_sensitive (self->priv->img_input_sound, FALSE);
        g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [INPUT_SOUND_MUTE], 0);
    } else {
        gtk_widget_set_sensitive (self->priv->img_input_sound, TRUE);
        g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [INPUT_SOUND_UNMUTE], 0);
    }
}

static void
tapiocaui_communication_send_message_cb (gpointer control,
                                         gpointer user_data)
{
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (user_data);
    const gchar *msg = gtk_entry_get_text (GTK_ENTRY (self->priv->input_message));
    if (g_utf8_strlen (msg, -1) > 0) {
        g_signal_emit (G_OBJECT (user_data), tapiocaui_communication_signals [SEND_MESSAGE], 0, msg);
        tapiocaui_communication_append_message (self, 0, _("you") , msg);
        gtk_entry_set_text (GTK_ENTRY (self->priv->input_message), "");
    }
}

void
tapiocaui_communication_append_message (TapiocaUICommunication *self,
                                        guint timestamp,
                                        const gchar *user,
                                        const gchar *msg)                                        
{
    char str_time[255];
    gchar *msg_str;
    time_t the_time = timestamp;
    GtkTextBuffer *text_buffer;
    GtkTextIter iter_begin;
    GtkTextIter iter_end;  
    GtkTextMark *mark_begin;

    text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->priv->txt_msglist));
    if ((self->priv->message_own == NULL) ||
        ((self->priv->message_own != NULL) &&
        (g_strcasecmp (self->priv->message_own, user) != 0))) {  
        self->priv->message_own = g_strdup (user);
        tapiocaui_debug ("create title");
        if (g_strcasecmp (user, _("you")) == 0) {
            msg_str = g_strdup_printf (_(" %s say:\n"), user);
        } else {
            msg_str = g_strdup_printf (_(" %s says:\n"), user);
        }
        gtk_text_buffer_get_end_iter (text_buffer, &iter_begin);
        mark_begin = gtk_text_buffer_create_mark  (text_buffer, "title_begin", &iter_begin, TRUE);
        gtk_text_buffer_get_iter_at_mark (text_buffer, &iter_end, mark_begin);
        gtk_text_buffer_insert (text_buffer, &iter_end, msg_str, -1);
        gtk_text_buffer_get_iter_at_mark (text_buffer, &iter_begin, mark_begin);        
        gtk_text_buffer_apply_tag_by_name (text_buffer, "bold", &iter_begin, &iter_end);
        tapiocaui_debug ("set color green");
        if (g_strcasecmp (user, _("you")) == 0) {
            gtk_text_buffer_apply_tag_by_name (text_buffer, "fg_green", &iter_begin, &iter_end);
        }        
        g_free (msg_str);
        gtk_text_buffer_delete_mark (text_buffer, mark_begin);        
    } 

    if (the_time == 0)
        time (&the_time);

    strftime (str_time, 254, "%H:%M", localtime (&the_time));
    
    //insert time
    tapiocaui_debug ("insert time");
    msg_str = g_strdup_printf ("\t[%s] ", str_time);
    gtk_text_buffer_get_end_iter (text_buffer, &iter_begin);
    mark_begin = gtk_text_buffer_create_mark  (text_buffer, "time_begin", &iter_begin, TRUE);
    gtk_text_buffer_get_iter_at_mark (text_buffer, &iter_end, mark_begin);
    gtk_text_buffer_insert (text_buffer, &iter_end, msg_str, -1);
    gtk_text_buffer_get_iter_at_mark (text_buffer, &iter_begin, mark_begin);        
    gtk_text_buffer_apply_tag_by_name (text_buffer, "fg_gray", &iter_begin, &iter_end);    
    g_free (msg_str);
    gtk_text_buffer_delete_mark (text_buffer, mark_begin);        

    //insert message
    tapiocaui_debug ("insert message");
    msg_str = g_strdup_printf ("%s\n", msg);
    gtk_text_buffer_get_end_iter (text_buffer, &iter_begin);
    mark_begin = gtk_text_buffer_create_mark  (text_buffer, "time_begin", &iter_begin, TRUE);
    gtk_text_buffer_get_iter_at_mark (text_buffer, &iter_end, mark_begin);
    gtk_text_buffer_insert (text_buffer, &iter_end, msg_str, -1);
    g_free (msg_str);
    gtk_text_buffer_delete_mark (text_buffer, mark_begin);        
   
    g_idle_add (tapiocaui_communication_auto_scroll, self);
}

void
tapiocaui_communication_closed_win_message (TapiocaUICommunication *self,
                                            const gchar *user)
{
    char str_time[255];
    gchar *msg_str;
    time_t the_time;
    GtkTextBuffer *text_buffer;
    GtkTextIter iter_begin;
    GtkTextIter iter_end;  
    GtkTextMark *mark_begin;

    time (&the_time);
    strftime (str_time, 254, "%H:%M", localtime (&the_time));
    
    text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self->priv->txt_msglist));

    //insert time
    tapiocaui_debug ("insert time");
    msg_str = g_strdup_printf ("[%s] %s has closed the conversation window\n", str_time, user);
    gtk_text_buffer_get_end_iter (text_buffer, &iter_begin);
    mark_begin = gtk_text_buffer_create_mark  (text_buffer, "time_begin", &iter_begin, TRUE);
    gtk_text_buffer_get_iter_at_mark (text_buffer, &iter_end, mark_begin);
    gtk_text_buffer_insert (text_buffer, &iter_end, msg_str, -1);
    gtk_text_buffer_get_iter_at_mark (text_buffer, &iter_begin, mark_begin);        
    gtk_text_buffer_apply_tag_by_name (text_buffer, "bold", &iter_begin, &iter_end);
    gtk_text_buffer_apply_tag_by_name (text_buffer, "fg_black", &iter_begin, &iter_end);    
    g_free (msg_str);
    gtk_text_buffer_delete_mark (text_buffer, mark_begin);        

    g_idle_add (tapiocaui_communication_auto_scroll, self);
}
    
static gboolean
tapiocaui_communication_auto_scroll (gpointer data)
{
    GtkAdjustment *vadj;
    gint newpos;
    GtkScrolledWindow *win;
    TapiocaUICommunication *self = TAPIOCAUI_COMMUNICATION (data);

    win = GTK_SCROLLED_WINDOW (gtk_widget_get_parent (self->priv->txt_msglist));
    if (win->vscrollbar_visible) {
        vadj = gtk_scrolled_window_get_vadjustment(win);
        newpos = vadj->upper - vadj->lower - vadj->page_size;
        gtk_adjustment_set_value(vadj, newpos);
        gtk_adjustment_value_changed(vadj);
    }
    return FALSE;
}
