/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtkvbox.h>
#include <gtk/gtktreeview.h>
#include <gtk/gtkliststore.h>
#include <gtk/gtkcellrenderer.h>
#include <gtk/gtkcellrenderertext.h>
#include <gtk/gtkcellrendererpixbuf.h>
#include <gtk/gtktreeselection.h>
#include "tapiocaui_list.h"

enum
{
    ITER_SELECTED,
    ITER_UNSELECTED,
    ITER_ACTIVATED,
    LAST_SIGNAL
};

struct _TapiocaUIListPrivate
{
    GtkTreeView *treeview;
    GtkTreePath *over_path;
    GtkTreeIter toplevel;
    GtkTreeSelection *selection;
    GdkColor *over_color;
    gboolean redraw;
};

typedef struct
{
    TapiocaUIList *self;
    gpointer user_data;
    TapiocaUIListCellDataFunc *data_func;
} RenderData;

static void      tapiocaui_list_class_init   (TapiocaUIListClass *k);
static void      tapiocaui_list_init         (TapiocaUIList *self);
static GObject  *tapiocaui_list_constructor  (GType type,
                                              guint n_construct_properties,
                                              GObjectConstructParam *construct_properties);
static void      tapiocaui_list_finalize     (GObject *object);
static void      tapiocaui_list_redraw       (TapiocaUIList *self);
static gboolean  tapiocaui_list_redraw_idle  (gpointer user_data);
static gboolean  tapiocaui_list_motion_cb    (GtkWidget *widget,
                                              GdkEventMotion *event,
                                              gpointer user_data);
static gboolean  tapiocaui_list_leave_cb     (GtkWidget *widget,
                                              GdkEventCrossing *event,
                                              gpointer user_data);
static gboolean  tapiocaui_list_selection_cb (GtkTreeSelection *selection,
                                              GtkTreeModel *model,
                                              GtkTreePath *path,
                                              gboolean path_currently_selected,
                                              gpointer data);
static void      tapiocaui_list_activated_cb (GtkTreeView *treeview,
                                              GtkTreePath *arg1,
                                              GtkTreeViewColumn *arg2,
                                              gpointer user_data);
static void      tapiocaui_list_render_item  (GtkTreeViewColumn *col,
                                              GtkCellRenderer *renderer,
                                              GtkTreeModel *model,
                                              GtkTreeIter *iter,
                                              gpointer user_data);

static guint tapiocaui_list_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

G_DEFINE_TYPE (TapiocaUIList, tapiocaui_list, GTK_TYPE_SCROLLED_WINDOW)

static void
tapiocaui_list_class_init (TapiocaUIListClass *k)
{
    GObjectClass *object_class = G_OBJECT_CLASS (k);
    parent_class = g_type_class_peek_parent (k);
    object_class->constructor = tapiocaui_list_constructor;
    object_class->finalize = tapiocaui_list_finalize;

    tapiocaui_list_signals[ITER_SELECTED] =
        g_signal_new ("iter-selected",
                      G_OBJECT_CLASS_TYPE (object_class),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIListClass, iter_selected),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_POINTER);
    tapiocaui_list_signals[ITER_UNSELECTED] =
        g_signal_new ("iter-unselected",
                      G_OBJECT_CLASS_TYPE (object_class),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIListClass, iter_unselected),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_POINTER);
    tapiocaui_list_signals[ITER_ACTIVATED] =
        g_signal_new ("iter-activated",
                      G_OBJECT_CLASS_TYPE (object_class),
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIListClass, iter_activated),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__POINTER,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_POINTER);
}

static void
tapiocaui_list_init (TapiocaUIList *self)
{
    self->priv = g_new0 (TapiocaUIListPrivate, 1);
}

static GObject *
tapiocaui_list_constructor (GType type,
                            guint n_construct_properties,
                            GObjectConstructParam *construct_properties)
{
    TapiocaUIList *self;
    TapiocaUIListClass *klass;
    GObjectClass *parent_class;
    GtkStyle *style;
    GdkColor gdk_color;

    klass = TAPIOCAUI_LIST_CLASS (g_type_class_peek (TAPIOCAUI_TYPE_LIST));

    parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
    self = TAPIOCAUI_LIST (parent_class->constructor (type, n_construct_properties,
                                                      construct_properties));

    self->priv->treeview = GTK_TREE_VIEW (gtk_tree_view_new ());

    self->priv->selection = gtk_tree_view_get_selection (self->priv->treeview);
    gtk_tree_selection_set_select_function (self->priv->selection,
                                            tapiocaui_list_selection_cb,
                                            self,
                                            NULL);

    style  = gtk_rc_get_style (GTK_WIDGET (self->priv->treeview));
    gdk_color = style->bg[GTK_STATE_SELECTED];

    self->priv->over_color = g_new0 (GdkColor, 1);
    self->priv->over_color->pixel  = 0;
    self->priv->over_color->red =  (65535 - gdk_color.red) / 2 + gdk_color.red;
    self->priv->over_color->green =  (65535 - gdk_color.green) / 2 + gdk_color.green;
    self->priv->over_color->blue = (65535 - gdk_color.blue) / 2 + gdk_color.blue;

    g_signal_connect_object (G_OBJECT (self->priv->treeview),
                             "motion-notify-event",
                             G_CALLBACK (tapiocaui_list_motion_cb),
                             self,
                             0);
    g_signal_connect_object (G_OBJECT (self->priv->treeview),
                             "leave-notify-event",
                             G_CALLBACK (tapiocaui_list_leave_cb),
                             self,
                             0);
    g_signal_connect_object (G_OBJECT (self->priv->treeview),
                             "row-activated",
                             G_CALLBACK (tapiocaui_list_activated_cb),
                             self,
                             0);

    gtk_tree_view_set_fixed_height_mode (self->priv->treeview, FALSE);

    self->priv->selection = gtk_tree_view_get_selection (self->priv->treeview);
    gtk_tree_selection_set_mode (self->priv->selection, GTK_SELECTION_SINGLE);
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (self->priv->treeview), FALSE);
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (self),
                                           GTK_WIDGET (self->priv->treeview));

    return G_OBJECT (self);
}

static void
tapiocaui_list_finalize (GObject *object)
{
    TapiocaUIList *self;
    g_return_if_fail (object != NULL);
    g_return_if_fail (TAPIOCAUI_IS_LIST (object));

    self = TAPIOCAUI_LIST (object);
    g_free (self->priv->over_color);
    g_free (self->priv);

    G_OBJECT_CLASS (parent_class)->finalize (object);
}

TapiocaUIList *
tapiocaui_list_new (void)
{
    TapiocaUIList *self;
    self = g_object_new (TAPIOCAUI_TYPE_LIST,
                         "hadjustment", NULL,
                         "vadjustment", NULL,
                         "hscrollbar_policy", GTK_POLICY_AUTOMATIC,
                         "vscrollbar_policy", GTK_POLICY_ALWAYS,
                         "shadow_type", GTK_SHADOW_IN,
                         NULL);
    return self;
}

static void
tapiocaui_list_redraw (TapiocaUIList *self)
{
    if (self->priv->redraw == FALSE) {
        g_idle_add (tapiocaui_list_redraw_idle, self);
        self->priv->redraw = TRUE;
    }
}

static gboolean
tapiocaui_list_redraw_idle (gpointer user_data)
{
    TapiocaUIList *self = (TapiocaUIList* ) user_data;
    gtk_tree_view_columns_autosize (self->priv->treeview);
    self->priv->redraw = FALSE;
    return FALSE;
}

static gboolean
tapiocaui_list_motion_cb (GtkWidget *widget,
                          GdkEventMotion *event,
                          gpointer user_data)
{
    GtkTreePath *path;
    TapiocaUIList *self = (TapiocaUIList* ) user_data;

    gtk_tree_view_get_path_at_pos (self->priv->treeview,
                                   event->x,
                                   event->y,
                                   &path,
                                   NULL,
                                   NULL,
                                   NULL);

    if (path != NULL) {
        if ((self->priv->over_path != NULL) &&
            (self->priv->over_path == path))
            return FALSE;

        if (self->priv->over_path != NULL) {
            gtk_tree_path_free (self->priv->over_path);
            self->priv->over_path = NULL;
        }
        self->priv->over_path = path;
        tapiocaui_list_redraw (self);
        return TRUE;
    } else {
        if (self->priv->over_path != NULL) {
            gtk_tree_path_free (self->priv->over_path);
            self->priv->over_path = NULL;
        }
        tapiocaui_list_redraw (self);
    }

    return FALSE;
}

static gboolean
tapiocaui_list_leave_cb (GtkWidget *widget,
                         GdkEventCrossing *event,
                         gpointer user_data)

{
    TapiocaUIList *self = TAPIOCAUI_LIST (user_data);
    if (self->priv->over_path != NULL) {
        gtk_tree_path_free (self->priv->over_path);
        self->priv->over_path = NULL;
        tapiocaui_list_redraw (self);
        return TRUE;
    }
    return FALSE;
}

void
tapiocaui_list_set_model (TapiocaUIList *self,
                          GtkTreeModel *model)
{
    gtk_tree_view_set_model (self->priv->treeview, model);
}

static void 
tapiocaui_list_render_item (GtkTreeViewColumn *col,
                            GtkCellRenderer *renderer,
                            GtkTreeModel *model,
                            GtkTreeIter *iter,
                            gpointer user_data)
{
    RenderData *data = (RenderData *) user_data;
    gboolean mouse_over;
    GtkTreePath *path = gtk_tree_model_get_path (model, iter);

    if ((data->self->priv->over_path != NULL) &&
        (!gtk_tree_path_compare (data->self->priv->over_path, path))) {
        mouse_over = TRUE;
        g_object_set (renderer,
                      "cell-background-gdk", data->self->priv->over_color,
                      NULL);
    } else {
        mouse_over = FALSE;
        g_object_set (renderer,
                      "cell-background-gdk", NULL,
                      NULL);
    }
    gtk_tree_path_free (path);
    data->data_func (col, renderer, model, iter, mouse_over, data->user_data);
}

GtkTreeSelection *
tapiocaui_list_get_selection (TapiocaUIList *self)
{
    return gtk_tree_view_get_selection (self->priv->treeview);
}

void
tapiocaui_list_append_collumn (TapiocaUIList *self,
                               GtkTreeViewColumn *column,
                               GtkCellRenderer *renderer,
                               TapiocaUIListCellDataFunc data_func,
                               gpointer user_data)
{
    RenderData *data = g_new0 (RenderData, 1);
    data->self = self;
    data->user_data = user_data;
    data->data_func = data_func;

    gtk_tree_view_column_set_cell_data_func (column, renderer,
                                             (GtkTreeCellDataFunc) tapiocaui_list_render_item,
                                             data,
                                             NULL);
    gtk_tree_view_append_column (self->priv->treeview, column);
}

static gboolean 
tapiocaui_list_selection_cb (GtkTreeSelection *selection,
                             GtkTreeModel *model,
                             GtkTreePath *path,
                             gboolean path_currently_selected,
                             gpointer user_data)
{
    TapiocaUIList *self = TAPIOCAUI_LIST (user_data);
    GtkTreeIter iter;

    model = gtk_tree_view_get_model (self->priv->treeview);
    gtk_tree_model_get_iter (model, &iter, path);

    if (path_currently_selected == FALSE) {
        g_signal_emit (G_OBJECT (self), tapiocaui_list_signals [ITER_SELECTED], 0, &iter);
    } else {
        g_signal_emit (G_OBJECT (self), tapiocaui_list_signals [ITER_UNSELECTED], 0, &iter);
    }
    tapiocaui_list_redraw (self);
    return TRUE;
}

static void 
tapiocaui_list_activated_cb (GtkTreeView *treeview,
                             GtkTreePath *path,
                             GtkTreeViewColumn *arg2,
                             gpointer user_data)
{
    TapiocaUIList *self = TAPIOCAUI_LIST (user_data);
    GtkTreeIter iter;
    GtkTreeModel *model;

    model = gtk_tree_view_get_model (self->priv->treeview);
    gtk_tree_model_get_iter (model, &iter, path);

    g_signal_emit (G_OBJECT (self), tapiocaui_list_signals [ITER_ACTIVATED], 0, &iter);
}
