/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glade/glade.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <tapioca/client/tapioca.h>
#include <tapioca/core/tapioca_enums.h>

#include "tapiocaui_new_buddy_dialog.h"
#include "tapiocaui_utils.h"

struct _TapiocaUINewBuddyDialogPrivate
{
    GtkWidget *entry_name;
    GtkWidget *entry_id;
    GtkWidget *btn_add;
    TapiocaUIBuddyData *edit_buddy;
    TapiocaUIDB *db;
};

static void     tapiocaui_new_buddy_dialog_class_init (TapiocaUINewBuddyDialogClass *k);
static void     tapiocaui_new_buddy_dialog_init       (TapiocaUINewBuddyDialog *self);
static void     tapiocaui_new_buddy_dialog_finalize   (GObject *object);
static gboolean tapiocaui_new_buddy_dialog_entry_cb   (GtkWidget *widget,
                                                       gpointer user_data);
static void     tapiocaui_new_buddy_dialog_reponse_cb (GtkDialog *dialog,
                                                       gint arg1,
                                                       gpointer user_data);

static GObjectClass *parent_class = NULL;

G_DEFINE_TYPE (TapiocaUINewBuddyDialog, tapiocaui_new_buddy_dialog, GTK_TYPE_DIALOG)

static void
tapiocaui_new_buddy_dialog_class_init (TapiocaUINewBuddyDialogClass *k)
{
    GObjectClass *object_class = G_OBJECT_CLASS (k);
    parent_class = g_type_class_peek_parent (k);
    object_class->finalize = tapiocaui_new_buddy_dialog_finalize;
}

static void
tapiocaui_new_buddy_dialog_init (TapiocaUINewBuddyDialog *self)
{
    GladeXML *xml;

    self->priv = g_new0 (TapiocaUINewBuddyDialogPrivate, 1);

    g_signal_connect (G_OBJECT (self),
                      "response",
                      G_CALLBACK (tapiocaui_new_buddy_dialog_reponse_cb),
                      self);

    gtk_dialog_set_default_response (GTK_DIALOG (self),
                                     GTK_RESPONSE_OK);

    gtk_dialog_add_button (GTK_DIALOG (self),
                           GTK_STOCK_CANCEL,
                           GTK_RESPONSE_CANCEL);

    self->priv->btn_add = gtk_dialog_add_button (GTK_DIALOG (self),
                                                 GTK_STOCK_OK,
                                                 GTK_RESPONSE_OK);
    gtk_widget_set_sensitive (self->priv->btn_add, FALSE);

    xml = glade_xml_new (tapiocaui_file ("new-buddy-dialog.glade"), 
                         "box_info", NULL);
    glade_xml_signal_autoconnect (xml);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (self)->vbox),
                       glade_xml_get_widget (xml, "box_info"));

    self->priv->entry_name = glade_xml_get_widget (xml, "entry_name");
    self->priv->entry_id = glade_xml_get_widget (xml, "entry_id");

    g_signal_connect (G_OBJECT (self->priv->entry_name),
                      "changed",
                      G_CALLBACK (tapiocaui_new_buddy_dialog_entry_cb),
                      self);

    g_signal_connect (G_OBJECT (self->priv->entry_id),
                      "changed",
                      G_CALLBACK (tapiocaui_new_buddy_dialog_entry_cb),
                      self);

    g_object_unref (G_OBJECT (xml));
}

static void
tapiocaui_new_buddy_dialog_finalize (GObject *object)
{
    TapiocaUINewBuddyDialog *self;

    g_return_if_fail (object != NULL);
    g_return_if_fail (TAPIOCAUI_IS_NEW_BUDDY_DIALOG (object));

    self = TAPIOCAUI_NEW_BUDDY_DIALOG (object);
    g_free (self->priv);

    G_OBJECT_CLASS (parent_class)->finalize (object);
}

GtkWidget *
tapiocaui_new_buddy_dialog_new (TapiocaUIDB *db,
                                gboolean edit_mode)
{
    TapiocaUINewBuddyDialog *self;
    self = g_object_new (TAPIOCAUI_TYPE_NEW_BUDDY_DIALOG, NULL);
    if (edit_mode)
        gtk_window_set_title (GTK_WINDOW (self), _("Edit Buddy"));
    else
        gtk_window_set_title (GTK_WINDOW (self), _("New Buddy"));
    self->priv->db = db;
    return GTK_WIDGET (self);
}

static void
tapiocaui_new_buddy_dialog_reponse_cb (GtkDialog *dialog,
                                       gint response_id,
                                       gpointer user_data)
{
    TapiocaUIBuddyData *buddy;
    TapiocaUINewBuddyDialog *self = (TapiocaUINewBuddyDialog *) user_data;

    if (response_id != GTK_RESPONSE_OK)
        return;

    /* if (!tapioca_is_valid_uri (gtk_entry_get_text (GTK_ENTRY (self->priv->entry_id)))) {
            GtkWidget *dialog;
            dialog = gtk_message_dialog_new (GTK_WINDOW (self),
                                             GTK_DIALOG_MODAL,
                                             GTK_MESSAGE_ERROR,
                                             GTK_BUTTONS_CLOSE,
                                             _("Invalid URI."));
            gtk_dialog_run (GTK_DIALOG (dialog));
            gtk_widget_destroy (dialog);
            return;            
    }*/
    
    if (self->priv->edit_buddy) {
        buddy = self->priv->edit_buddy;
    } else {
        buddy = tapiocaui_db_find_buddy_by_id (self->priv->db, TAPIOCAUI_DB_ENTRY_BUDDY,
                                               gtk_entry_get_text (GTK_ENTRY (self->priv->entry_id)));
        if (buddy) {
            GtkWidget *dialog;
            dialog = gtk_message_dialog_new (GTK_WINDOW (self),
                                             GTK_DIALOG_DESTROY_WITH_PARENT,
                                             GTK_MESSAGE_ERROR,
                                             GTK_BUTTONS_CLOSE,
                                             _("User already exists."));
            gtk_dialog_run (GTK_DIALOG (dialog));
            return;
        }
        buddy = tapiocaui_db_entry_new (TAPIOCAUI_DB_ENTRY_BUDDY);
    }

    buddy->id = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->priv->entry_id)));
    buddy->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->priv->entry_name)));
    
    // TODO
    // Create this field on dialog.
    buddy->group = g_strdup ("");

    if (!self->priv->edit_buddy) {
        tapiocaui_db_insert_buddy (self->priv->db, buddy);
    } else {
        tapiocaui_db_notify_change (self->priv->db);
    }
    tapiocaui_db_commit (self->priv->db);
}

void
tapiocaui_new_buddy_dialog_set_edit_buddy (TapiocaUINewBuddyDialog *self,
                                           TapiocaUIBuddyData *buddy)
{
    g_return_if_fail (buddy != NULL);
    self->priv->edit_buddy = buddy;
    gtk_entry_set_text (GTK_ENTRY (self->priv->entry_id), buddy->id);
    gtk_entry_set_text (GTK_ENTRY (self->priv->entry_name), buddy->name);
}

static gboolean
tapiocaui_new_buddy_dialog_entry_cb (GtkWidget *widget,
                                     gpointer user_data)
{
    TapiocaUINewBuddyDialog *self = (TapiocaUINewBuddyDialog *) user_data;
    gchar *entry_id;
    gchar *entry_name;

    entry_id = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->priv->entry_id)));
    entry_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->priv->entry_name)));
    
    g_strstrip (entry_id);
    g_strstrip (entry_name);
    
    if ((g_utf8_strlen (entry_id, -1) > 0) &&
        (g_utf8_strlen (entry_name, -1) > 0)) {
        gtk_widget_set_sensitive (GTK_WIDGET (self->priv->btn_add), TRUE);
    } else {
        gtk_widget_set_sensitive (GTK_WIDGET (self->priv->btn_add), FALSE);
    }

    g_free (entry_id);
    g_free (entry_name);

    return  FALSE;
}

const gchar *
tapiocaui_new_buddy_dialog_get_id (TapiocaUINewBuddyDialog *self)
{
    g_return_val_if_fail (self != NULL, NULL);
    return gtk_entry_get_text (GTK_ENTRY (self->priv->entry_id));
}
